# Cronograma: Introdução à Ciência da Computação em C

## Capítulo 1: Como funciona um computador

### Resumo

- [x] Descrever as principais partes de um computador

  - [x] Unidade Central de Processamento (microprocessadores)

  - [x] Memória

  - [x] Dispositivos de Entrada e Saída

- [x] Diferenciar software e hardware

- [x] Definir o que é um compilador

- [x] Descrever o que é um algoritmo e dar exemplos práticos

## Capítulo 2: Um primeiro programa em C

### Resumo

- [x] Descrever o ambiente em que os seus programas serão desenvolvidos.

- [x] Escrever um programa em C que recebe dados do teclado e imprime dados na tela, usando as funções `scanf` e `printf`.

- [x] Compilar um programa em C usando o `gcc`.

- [x] Executar o programa após a compilação.

## Capítulo 3: Fundamentos

### Resumo

- [x] Descrever e utilizar variáveis.

- [x] Descrever a precedência dos operadores e como isso afeta o resultado das expressões aritméticas.

- [x] Utilizar expressões aritméticas e relacionais, e prever seus resultados para entradas conhecidas.

- [x] Utilizar comandos de leitura, impressão e atribuição.

## Capítulo 4: Comandos de repetição: *while*

### Resumo

- [x] Utilizar comandos de repetição na resolução de problemas computacionais.

- [x] Definir condições iniciais e de parada para o comando `while`.

- [x] Simular o processamento do comando `while`.

## Capítulo 5: Comandos de seleção simples e composta

### Resumo

- [x] Utilizar o comando `if`.

- [x] Utilizar o comando `if-else`

- [x] Identificar situações onde é necessário utilizar o comando `if-else` e situações onde é suficiente utilizar o comando `if`.

- [x] Simular o processamento dos comandos `if` e `if-else` em um programa.

## Capítulo 6: Sobre a divisão e resto de números inteiros

### Resumo

- [x] Calcular o resultado de expressões aritméticas que contenham os operadores de divisão e resto com números inteiros.

- [x] Utilizar os operadores de divisão e resto de números inteiros em seus programas.

## Capítulo 7: Mais detalhes sobre a linguagem C

### Resumo

- [x] Identificar blocos, situações que necessitam a utilização de chaves em seu programa, e situações onde as chaves podem ser omitidas.

- [x] Simular e utilizar abreviaturas do comando de atribuição (como += e -=).

- [x] Simular e utilizar atribuições múltiplas e no instante da declaração.

- [x] Definir constantes e utilizá-las em seus programas.

## Capítulo 8: Operadores lógicos

### Resumo

- [x] Calcular o valor de expressões aritméticas, relacionais e lógicas.

- [x] Utilizar operadores lógicos para a construção de condições complexas em seus programas.

## Capítulo 9: Dicas de programação

### Resumo

- [x] Definir o conceito de padrão em computação

- [x] Utilizar os padrões computacionais: padrão sequência númerica lida, padrão sequência numérica gerada, padrão sequência numérica selecionada e padrão sequência numérica alternadamente selecionada.

## Capítulo 10: Comando de repetição: *for*

### Resumo

- [x] Descrever a sintaxe do comando`for`.

- [x] Simular o funcionamento do comando `for`.

- [x] Utilizar o comando `for` em programas.

## Capítulo 11: Repetições encaixadas

### Resumo

- [x] Simular programas com repetições encaixadas.

- [x] Utilizar repetições encaixadas em seus programas.

## Capítulo 12: Indicador de passagem

### Resumo

- [x] Descrever o funcionamento de indicadores de passagem.

- [x] Identificar situações onde indicadores de passagem podem ser utilizados.

- [x] Utilizar indicadores de passagem em seus programas.

## Capítulo 13: Números reais do tipo *float*

### Resumo

- [x] Declarar, ler e imprimir variáveis do tipo float.

- [x] Calcular o valor de expressões aritméticas envolvendo reais.

- [x] Utilizar variáveis reais em programas.

## Capítulo 14: Fórmula de recorrência e séries (Somas Infinitas)

### Resumo

- [x] Descrever o que são fórmulas de recorrência.

- [x] Descrever o que são erro absoluto e relativo.

- [x] Escrever programas em C a partir de fórmulas de recorrência.

## Capítulo 15: Introdução às funções

### Resumos

- [x] Justificar as vantagens do uso de funções em programas.

- [x] Identificar em seus programas oportunidades para criação e uso de funções.

- [x] Definir os parâmetros que a função precisa receber (parâmetros de entrada) e o que a função precisa devolver (o tipo do valor devolvido pela função).

## Capítulo 16: Definição e uso de funções em programas

### Resumo

- [x] Escrever o protótipo de uma função.

- [x] Escrever o corpo de uma função.

- [x] Utilizar funções em seus programas.

- [x] Simular a execução de programas com funções.

## Capítulo 17: Funções e ponteiros

### Resumo

- [x] Descrever como a informação é armazenada na memória do computador, e a diferença entre conteúdo e endereço de uma posição de memória.

- [x] Definir o que é um ponteiro (ou apontador), e para que ele é utilizado.

- [x] Declarar ponteiros.

- [x] Utilizar ponteiros como parâmetros de funcões e em programas.

## Capítulo 18: Vetores

### Resumo

- [x] Descrever o que são vetores na linguagem C.

- [x] Declarar vetores.

- [x] Como acessar elementos de um vetor e percorrer um vetor.

- [x] Utilizar vetores para resolver problemas computacionais.

## Capítulo 19: Vetores, ponteiros e funções

### Resumo

- [x] Descrever como os vetores são armazenados na memória.

- [x] Descrever a relação entre vetores e ponteiros.

- [x] Utilizar vetores como parâmetros de funções.

## Capítulo 20: Caracteres: *char*

### Resumo

- [x] Declarar, ler e imprimir variáveis do tipo `char`.

- [x] Descrever como caracteres são representados no computador e porque eles podem ser utilizados em expressões.

- [x] Utilizar variáveis do tipo `char` em programas.

## Capítulo 21: *Strings*

### Resumo

- [x] Descrever o que são `strings`.

- [x] Descrever a distinção entre `strings` e vetores de caracteres.

- [x] Ler e imprimir `strings`.

- [x] Usar recursos da `string.h`[^1].

- [x] Utilizar `strings` em seus programas.

[^1]: Utilizou-se da biblioteca `wchar.h`, adequada a realizar operações com `wchar_t`. Estas variáveis do tipo `char` com consumo de memória adicional para armazenarem uma amplitude de caracteres acentuados presentes no idioma português.

## Capítulo 22: Matrizes

### Resumo

- [ ] Descrever o que são matrizes em C.

- [ ] Declarar matrizes.

- [ ] Como acessar elementos de uma matriz e percorrer uma matriz.

- [ ] Utilizar matrizes para resolver problemas computacionais.



## Capítulo 23: Matrizes, Ponteiros e Funções

### Resumo

- [ ] Descrever como matrizes são armazenadas na memória.

- [ ] Descrever a relação entre matrizes e ponteiros.

- [ ] Utilizar matrizes como parâmetros de funções.



## Capítulo 24: Algoritmos de busca

### Resumo

- [ ] Definir o problema de busca de um elemento em um vetor.

- [ ] Descrever o que é busca sequencial.

- [ ] Descrever a vantagem de usar sentinela em busca sequencial.
